using System.Runtime.InteropServices;


namespace EMS
{
	[StructLayoutAttribute(LayoutKind.Sequential,
	CharSet=CharSet.Auto)]
	public struct Complex
		{
		public double re;
		public double im;
		}	

	public class CxTranslator
	{
		//EvaluateEqu
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static  Complex EvaluateEqu(string exp, Complex [] arg); 
		//Evaluate
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static Complex Evaluate(string arg, string exp, Complex [] argVal); 
		//BuildEqu
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static  void BuildEqu(string mExp);
		//Build
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static  void Build(string arg, string exp);
		//CalcFor
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public   extern static  Complex CalcFor(Complex [] arg);

		//Delegate, AddFunction & RemoveFunction 
		public delegate  Complex  CallBackUserFunction(ref Complex val);
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static bool AddFunction(string funName,CallBackUserFunction fn);
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static bool RemoveFunction(string exp);
	
		//AddConst & RemoveConst
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static bool AddConst(string exp, Complex val);
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public extern static bool RemoveConst(string exp);


		// Declaring Delegate for the errors handle Callback Function
		public delegate void CallBackDelegate(string err);
		[DllImport("CxTranslator.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Ansi)]
		public static extern void  SetErrorHandle(CallBackDelegate dl);

	
		public static void SetEquTranslatorException()
		{
			CallBackDelegate errrBkCallFunc = new CallBackDelegate(RiseException);			
			SetErrorHandle(errrBkCallFunc);
		}

		static void RiseException(string err)
		{
			throw new System.Exception(err);
		}

	}


	
	/// ////////////////////////////////////////////////////////////////
	//Unicode impl of CxTranslator
	public class CxTranslatorW
	{
		//EvaluateEqu
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static  Complex EvaluateEqu(string exp, Complex [] arg); 
		//Evaluate
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static Complex Evaluate(string arg, string exp, Complex [] argVal); 
		//BuildEqu
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static  void BuildEqu(string mExp);
		//Build
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static  void Build(string arg, string exp);
		//CalcFor
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public   extern static  Complex CalcFor(Complex [] arg);

		//Delegate, AddFunction & RemoveFunction 
		public delegate  Complex  CallBackUserFunction(ref Complex val);
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static bool AddFunction(string funName,CallBackUserFunction fn);
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static bool RemoveFunction(string exp);
	
		//AddConst & RemoveConst
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static bool AddConst(string exp, Complex val);
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public extern static bool RemoveConst(string exp);


		// Declaring Delegate for the errors handle Callback Function
		public delegate void CallBackDelegate(string err);
		[DllImport("CxTranslatorW.dll",CallingConvention=CallingConvention.StdCall,
			 CharSet=CharSet.Unicode)]
		public static extern void  SetErrorHandle(CallBackDelegate dl);

	
		public static void SetEquTranslatorException()
		{
			CallBackDelegate errrBkCallFunc = new CallBackDelegate(RiseException);			
			SetErrorHandle(errrBkCallFunc);
		}

		static void RiseException(string err)
		{
			throw new System.Exception(err);
		}

	}
}

